(************************************************************************)
(* cctasksr-sig.sml                                                     *)
(*                                                                      *)
(* Description:                                                         *)
(*     Simulate the ConCert architecture using CML.  See below for      *)
(*     a discussion of the interface, and see cctasksr.sml for          *)
(*     for a discussion about the implementation.                       *)
(*                                                                      *)
(* Author:                                                              *)
(*     Evan Chang                                                       *)
(************************************************************************)

(* $Id: cctasksr-sig.sml,v 1.1.1.1 2002/05/15 22:00:06 bechang_conductor Exp $ *)

(************************************************************************)
(* signature CCTASKSR                                                   *)
(*                                                                      *)
(*   Interface:                                                         *)
(*                                                                      *)
(*     Because this simulation is implemented in SML, we will have to   *)
(*     will have to sacrifice some adherence to the specification       *)
(*     we would like to have in a true extension.  The biggest          *)
(*     difference between the interface given below and the desired     *)
(*     interface is that the code and arguments provided by the         *)
(*     user may not be polymorphic (the type of the function is fixed   *)
(*     to env -> result).  This is because we must aggregate all        *)
(*     such pieces of code into some data structure to support          *)
(*     restart.  It would be possible to allow the argument to          *)
(*     polymorphic, but it is unclear what that buys us and we believe  *)
(*     this way makes it more clear the limitation.                     *)
(*                                                                      *)
(*     The other difference is the simulation is started with an        *)
(*     initial task using 'startSim' rather than the initial task       *)
(*     being started at the command-line.                               *)
(*                                                                      *)
(*     Note that events are introduced only through recvResultEvt.      *)
(*     The 'wrap' function is the only way to create events with type   *)
(*     other than 'result event'.                                       *)
(*                                                                      *)
(************************************************************************)
signature CCTASKSR =
sig

  type env
  type result

  type task
  exception InvalidTask

  type 'a event

  (* Start a task  *)
  val inject : bool -> (env -> result) * env -> task

  (* Start a disabled tasks *)
  val enable : task -> unit
  
  (* Get result of a task *)
  val recvResult : task -> result
  val recvResultEvt : task -> result event

  (* A task is no longer needed *)
  val forget : task -> unit

  (* Exit early *)
  val exit : result -> 'a

  (* Event Combinators *)
  val sync   : 'a event -> 'a
  val choose : 'a event list -> 'a event
  val select : 'a event list -> 'a
  val wrap   : 'a event * ('a -> 'b) -> 'b event
  val guard  : (unit -> 'a event) -> 'a event

  (* Query the status of a task *)
  val status : task -> CCStatus.status

  (* Start Simulation *)
  val startSim : real -> (env -> result) * env -> result

end; (* signature CCTASKSR *)
