(************************************************************************)
(* mergesort.sml                                                        *)
(*                                                                      *)
(* Description:                                                         *)
(*    A simple implementation of mergesort to try-out the ConCert       *)
(*    simulator.                                                        *) 
(*                                                                      *)
(* Author:                                                              *)
(*    Evan Chang                                                        *)
(************************************************************************)

(* $Id: mergesort.sml,v 1.1.1.1 2002/05/15 22:00:06 bechang_conductor Exp $ *)


functor MergeSort(structure Comparable : sig
                                           type t
                                           val compare : t * t -> order
                                         end)
:> MERGESORT where type t = Comparable.t =
struct
  
  type t = Comparable.t

  (* Point at which we stop parallelizing subproblems *)
  val PAR_CUTOFF = 5

  structure MST =
  struct
    type env = t list
    type result = t list
  end (* structure MergeSort.MST *)

  structure Tasks = CCTasksRFn(structure T = MST)

  (* mergesort : bool -> real -> t list -> t list *)
  fun mergesort parallel failureParam l =
      let

        (* mergesort' : t list -> t list *)
        fun mergesort' (nil) = nil
          | mergesort' (l as [x]) = l
          | mergesort' (l) =
            let
              (* partition : int * t list -> t list * t list *)
              fun partition (_, nil) = (nil, nil)
                | partition (0, k as x::t) = (nil, k)
                | partition (n, x::t) =
                  let
                    val (l',k') = partition (n-1, t)
                  in
                    (x::l', k')
                  end
      
              (* merge : t list * t list -> t list *)
              fun merge (l, nil) = l
                | merge (nil, k) = k
                | merge (l as x::l', k as y::k') =
                  (case Comparable.compare (x,y) of
                     LESS    => x::(merge (l',k))
                   | EQUAL   => x::y::(merge (l',k'))
                   | GREATER => y::(merge (l,k')))
      
              val len = List.length l
              val (lt,rt) = partition (len div 2, l)
            in
              if (not parallel) orelse (len <= PAR_CUTOFF) then
                merge (mergesort' lt, mergesort' rt)
              else
                let
                  open Tasks
      
                  (* Start sorting each partition *)
                  val tid1 = inject true (mergesort', lt)
                  val tid2 = inject true (mergesort', rt)
      
                  val (sortlt, sortrt) = select [
                    wrap (recvResultEvt tid1,
                          fn sortlt => (sortlt, recvResult tid2)),
                    wrap (recvResultEvt tid2,
                          fn sortrt => (recvResult tid1, sortrt))
                  ]
                in
                  merge (sortlt, sortrt)
                end
            end

      in
        if parallel then Tasks.startSim failureParam (mergesort', l)
        else mergesort' l
      end
end; (* functor MergeSort *)
