(************************************************************************)
(* mergesort_relax.sml                                                  *)
(*                                                                      *)
(* Description:                                                         *)
(*    A simple implementation of mergesort to try-out the ConCert       *)
(*    simulator using relax.                                            *) 
(*                                                                      *)
(* Author:                                                              *)
(*    Evan Chang                                                        *)
(************************************************************************)

(* $Id: mergesort_relax.sml,v 1.1.1.1 2002/05/15 22:00:06 bechang_conductor Exp $ *)


functor MergeSortRelax(structure Comparable : sig
                                                type t
                                                val compare : t * t -> order
                                              end)
:> MERGESORT where type t = Comparable.t =
struct
  
  type t = Comparable.t

  (* Point at which we stop parallelizing subproblems *)
  val PAR_CUTOFF = 5

  structure MST =
  struct
    type env = t list
    type result = t list
  end (* structure MergeSort.MST *)

  structure Tasks = CCTasksFn(structure T = MST)

  (* mergesort : bool -> real -> t list -> t list *)
  fun mergesort parallel failureParam l =
      let

        (* mergesort' : t list -> t list *)
        fun mergesort' (nil) = nil
          | mergesort' (l as [x]) = l
          | mergesort' (l) =
            let
	      (* partition : int * int list
                               -> int list * int list * int list *)
	      fun partition (m, l) =
		  let
		    fun partition' (_, nil) = (nil, nil)
		      | partition' (0, k as x::t) = (nil, k)
		      | partition' (n, x::t) =
			let
			  val (l',k') = partition' (n-1, t)
			in
			  (x::l', k')
			end

		    val (ll,lr)   = partition' (m, l)
		    val (lrl,lrr) = partition' (m, lr)
		  in
		    (ll,lrl,lrr)
		  end

              (* merge : t list * t list -> t list *)
              fun merge (l, nil) = l
                | merge (nil, k) = k
                | merge (l as x::l', k as y::k') =
                  (case Comparable.compare (x,y) of
                     LESS    => x::(merge (l',k))
                   | EQUAL   => x::y::(merge (l',k'))
                   | GREATER => y::(merge (l,k')))
      
              val len = List.length l
              val (lt,md,rt) = partition (len div 3, l)
            in
              if (not parallel) orelse (len <= PAR_CUTOFF) then
                merge (mergesort' lt,
                       merge (mergesort' md, mergesort' rt))
              else
                let
                  open Tasks
      
                  (* Start sorting each partition *)
                  val t1 = inject true (mergesort', lt)
                  val t2 = inject true (mergesort', md)
                  val t3 = inject true (mergesort', rt)

		  (* Get the results of the three child tasks.  Start merging
		     when two sorted lists have been received. *)
		  val (sort1, sort2) =
		    let
		      val (a, rest) = relax [ t1, t2, t3 ]
		      val (b, [last]) = relax rest
		    in
		      (merge (a,b), sync last)
		    end
                in
                  merge (sort1, sort2)
                end
            end

      in
        if parallel then Tasks.startSim failureParam (mergesort', l)
        else mergesort' l
      end
end; (* functor MergeSort *)
