(************************************************************************)
(* test-mergesort.sml                                                   *)
(*                                                                      *)
(* Description:                                                         *)
(*    Test out the mergesort routine.                                   *)
(*                                                                      *)
(*    To run, call:                                                     *)
(*         MSTester.runTest : real -> int -> int list                   *)
(*      or MSRelaxTester.runTest : real -> int -> int list              *)
(*                                                                      *)
(*      The first parameter is the failure parameter to the simulator   *)
(*      and the second parameter is the test number to run.             *)
(*                                                                      *)
(* Author:                                                              *)
(*    Evan Chang                                                        *)
(************************************************************************)

(* $Id: test-mergesort.sml,v 1.1.1.1 2002/05/15 22:00:06 bechang_conductor Exp $ *)



(*********************************************************************)
(* MODIFIABLE                                                        *)
(*                                                                   *)
(* Modify these to change the amount of output given by SML/NJ.      *)
(* Change log-level to "Logger.DEBUG" for even more verbose output.  *)
(*********************************************************************)
SMLofNJ.Internals.GC.messages false;
Compiler.Control.Print.printLength := 100;
Logger.setLogLevel Logger.NOTE;




(*********************************************************************)
(* DO NOT NEED TO MODIFY BELOW THIS.                                 *)
(*********************************************************************)
structure IntCompare =
struct
  type t = int
  val compare = Int.compare
end; (* structure IntCompare *)

structure IntMergeSort
:> MERGESORT where type t = int
= MergeSort(structure Comparable = IntCompare)

structure IntMergeSortRelax
:> MERGESORT where type t = int
= MergeSortRelax(structure Comparable = IntCompare)

(* The "Tester" functor *)
functor MergeSortTestFn(MS : MERGESORT where type t = int) =
struct

  val t0 = []
  val t1 = [2, 1, 4, 3]
  val t2 = [3, 1, 5, 2, 4, 6, 7, 10, 9, 8]
  val t3 = List.tabulate (20, fn i => (i*61) mod 20)

  val tests = Vector.fromList [ t0, t1, t2, t3 ]

  fun runTest failureParam i =
        MS.mergesort true failureParam (Vector.sub (tests, i))

end; (* structure MergeSortTest *)

(* The instantiated "Tester" structures *)
structure MSTester = MergeSortTestFn(IntMergeSort)
structure MSRelaxTester = MergeSortTestFn(IntMergeSortRelax)
