(************************************************************************)
(* logger.sml                                                           *)
(*                                                                      *)
(* Description:                                                         *)
(*     A very simple interface for logging messages.                    *)
(*                                                                      *)
(* Author:                                                              *)
(*     Evan Chang                                                       *)
(*                                                                      *)
(* $Id: logger.sml,v 1.1.1.1 2002/05/15 22:00:06 bechang_conductor Exp $    *)
(************************************************************************)


signature LOGGER =
sig
  
  type level
  exception InvalidLevel

  val DEBUG : level
  val NOTE  : level
  val WARN  : level
  val ERR   : level
  val FATAL : level

  val levelToString : level -> string

  val setLogLevel : level -> unit
  val getLogLevel : unit -> level

  val setLogger : (level -> string -> unit) -> unit
  val getLogger : unit -> (level -> string -> unit)

  val log : level -> string -> unit

end; (* signature LOGGER *)

structure Logger =
struct

  type level = int
  exception InvalidLevel

  val DEBUG = 40
  val NOTE  = 30
  val WARN  = 20
  val ERR   = 10 
  val FATAL = 0

  fun levelToString lvl = 
      if      lvl=DEBUG then "DEBUG"
      else if lvl=NOTE  then "NOTE"
      else if lvl=WARN  then "WARNING"
      else if lvl=ERR   then "ERROR"
      else if lvl=FATAL then "FATAL"
      else raise InvalidLevel

  val DEFAULT_LOG_LEVEL = 10
  val logLevel = ref DEFAULT_LOG_LEVEL

  fun setLogLevel lvl = (logLevel := lvl)
  fun getLogLevel ()  = !logLevel

  fun DEFAULT_LOGGER lvl str =
      if lvl <= (getLogLevel ()) then
        TextIO.print ((levelToString lvl) ^ "\t" ^ str)
      else
        ()
  val logger = ref DEFAULT_LOGGER

  fun setLogger f  = (logger := f)
  fun getLogger () = (!logger)

  fun log x = (getLogger ()) x

end; (* structure Logger *)
